const express = require('express');

const KarusersService = require('../services/karusers');
const KarusersDBApi = require('../db/api/karusers');
const wrapAsync = require('../helpers').wrapAsync;

const router = express.Router();

/**
 *  @swagger
 *  components:
 *    schemas:
 *      Karusers:
 *        type: object
 *        properties:

 *          name:
 *            type: string
 *            default: name
 *          surname:
 *            type: string
 *            default: surname
 *          address_line1:
 *            type: string
 *            default: address_line1
 *          address_line2:
 *            type: string
 *            default: address_line2
 *          city:
 *            type: string
 *            default: city
 *          state:
 *            type: string
 *            default: state
 *          country:
 *            type: string
 *            default: country
 *          pincode:
 *            type: string
 *            default: pincode
 *          gothram:
 *            type: string
 *            default: gothram
 *          nakshatram:
 *            type: string
 *            default: nakshatram
 *          email_id:
 *            type: string
 *            default: email_id

 *          
 */

/**
 *  @swagger
 * tags:
 *   name: Karusers
 *   description: The Karusers managing API
 */

/**
 *  @swagger
 *  /api/karusers:
 *    post:
 *      security:
 *        - bearerAuth: []
 *      tags: [Karusers]
 *      summary: Add new item
 *      description: Add new item
 *      requestBody:
 *        required: true
 *        content:
 *          application/json:
 *            schema:
 *              properties:
 *                data:
 *                  description: Data of the updated item
 *                  type: object
 *                  $ref: "#/components/schemas/Karusers"
 *      responses:
 *        200:
 *          description: The item was successfully added
 *          content:
 *            application/json:
 *              schema:
 *                $ref: "#/components/schemas/Karusers"
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        405:
 *          description: Invalid input data
 *        500:
 *          description: Some server error
 */

router.post('/', async (req, res) => {
  await KarusersService.create(
    req.body.data,
    req.currentUser,
    true,
    req.headers.referer,
  );
  const payload = true;

  res.status(200).send(payload);
});

/**
 *  @swagger
 *  /api/karusers/{id}:
 *    put:
 *      security:
 *        - bearerAuth: []
 *      tags: [Karusers]
 *      summary: Update the data of the selected item
 *      description: Update the data of the selected item
 *      parameters:
 *        - in: path
 *          name: id
 *          description: Item ID to update
 *          required: true
 *          schema:
 *            type: string
 *      requestBody:
 *        description: Set new item data
 *        required: true
 *        content:
 *          application/json:
 *            schema:
 *              properties:
 *                id:
 *                  description: ID of the updated item
 *                  type: string
 *                data:
 *                  description: Data of the updated item
 *                  type: object
 *                  $ref: "#/components/schemas/Karusers"
 *              required:
 *                - id
 *      responses:
 *        200:
 *          description: The item data was successfully updated
 *          content:
 *            application/json:
 *              schema:
 *                $ref: "#/components/schemas/Karusers"
 *        400:
 *          description: Invalid ID supplied
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        404:
 *          description: Item not found
 *        500:
 *          description: Some server error
 */

router.put(
  '/:id',
  wrapAsync(async (req, res) => {

    await KarusersService.update(req.body.data, req.body.id, req.currentUser);
    const payload = true;
    res.status(200).send(payload);
  }),
);

/**
 * @swagger
 *  /api/karusers/{id}:
 *    delete:
 *      security:
 *        - bearerAuth: []
 *      tags: [Karusers]
 *      summary: Delete the selected item
 *      description: Delete the selected item
 *      parameters:
 *        - in: path
 *          name: id
 *          description: Item ID to delete
 *          required: true
 *          schema:
 *            type: string
 *      responses:
 *        200:
 *          description: The item was successfully deleted
 *          content:
 *            application/json:
 *              schema:
 *                $ref: "#/components/schemas/Karusers"
 *        400:
 *          description: Invalid ID supplied
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        404:
 *          description: Item not found
 *        500:
 *          description: Some server error
 */

router.delete(
  '/:id',
  wrapAsync(async (req, res) => {
    await KarusersService.remove(req.params.id, req.currentUser);
    const payload = true;
    res.status(200).send(payload);
  }),
);

/**
 *  @swagger
 *  /api/karusers:
 *    get:
 *      security:
 *        - bearerAuth: []
 *      tags: [Karusers]
 *      summary: Get all karusers
 *      description: Get all karusers
 *      responses:
 *        200:
 *          description: Karusers list successfully received
 *          content:
 *            application/json:
 *              schema:
 *                type: array
 *                items:
 *                  $ref: "#/components/schemas/Karusers"
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        404:
 *          description: Data not found
 *        500:
 *          description: Some server error
 */

router.get(
  '/',
  wrapAsync(async (req, res) => {

    const payload = await KarusersDBApi.findAll(req.query);

    res.status(200).send(payload);
  }),
);

router.get('/autocomplete', async (req, res) => {

  const payload = await KarusersDBApi.findAllAutocomplete(
    req.query.query,
    req.query.limit,
  );

  res.status(200).send(payload);
});

/**
 * @swagger
 *  /api/karusers/{id}:
 *    get:
 *      security:
 *        - bearerAuth: []
 *      tags: [Karusers]
 *      summary: Get selected item
 *      description: Get selected item
 *      parameters:
 *        - in: path
 *          name: id
 *          description: ID of item to get
 *          required: true
 *          schema:
 *            type: string
 *      responses:
 *        200:
 *          description: Selected item successfully received
 *          content:
 *            application/json:
 *              schema:
 *                $ref: "#/components/schemas/Karusers"
 *        400:
 *          description: Invalid ID supplied
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        404:
 *          description: Item not found
 *        500:
 *          description: Some server error
 */

router.get(
  '/:id',
  wrapAsync(async (req, res) => {
    try{
      const payload = await KarusersDBApi.findBy({ id: req.params.id });
      res.status(200).send(payload);
    } catch (err) {
      console.log(err);
    }
  }),
);

router.use('/', require('../helpers').commonErrorHandler);

module.exports = router;
