const express = require('express');

const Pujari_applicationsService = require('../services/pujari_applications');
const Pujari_applicationsDBApi = require('../db/api/pujari_applications');
const wrapAsync = require('../helpers').wrapAsync;

const router = express.Router();

/**
 *  @swagger
 *  components:
 *    schemas:
 *      Pujari_applications:
 *        type: object
 *        properties:

 *          name:
 *            type: string
 *            default: name
 *          surname:
 *            type: string
 *            default: surname
 *          qualification:
 *            type: string
 *            default: qualification
 *          address:
 *            type: string
 *            default: address
 *          email_id:
 *            type: string
 *            default: email_id
 *          phone_number:
 *            type: string
 *            default: phone_number
 *          city:
 *            type: string
 *            default: city

 *          experience_yrs:
 *            type: integer
 *            format: int64

 *          
 *          
 *          
 */

/**
 *  @swagger
 * tags:
 *   name: Pujari_applications
 *   description: The Pujari_applications managing API
 */

/**
 *  @swagger
 *  /api/pujari_applications:
 *    post:
 *      security:
 *        - bearerAuth: []
 *      tags: [Pujari_applications]
 *      summary: Add new item
 *      description: Add new item
 *      requestBody:
 *        required: true
 *        content:
 *          application/json:
 *            schema:
 *              properties:
 *                data:
 *                  description: Data of the updated item
 *                  type: object
 *                  $ref: "#/components/schemas/Pujari_applications"
 *      responses:
 *        200:
 *          description: The item was successfully added
 *          content:
 *            application/json:
 *              schema:
 *                $ref: "#/components/schemas/Pujari_applications"
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        405:
 *          description: Invalid input data
 *        500:
 *          description: Some server error
 */

router.post('/', async (req, res) => {
  try{
    await Pujari_applicationsService.create(
      req.body.data,
      req.currentUser,
      true,
      req.headers.referer,
    );
    const payload = true;

    res.status(200).send(payload);
  } catch (err) {
    console.log(err);
  }
});

/**
 *  @swagger
 *  /api/pujari_applications/{id}:
 *    put:
 *      security:
 *        - bearerAuth: []
 *      tags: [Pujari_applications]
 *      summary: Update the data of the selected item
 *      description: Update the data of the selected item
 *      parameters:
 *        - in: path
 *          name: id
 *          description: Item ID to update
 *          required: true
 *          schema:
 *            type: string
 *      requestBody:
 *        description: Set new item data
 *        required: true
 *        content:
 *          application/json:
 *            schema:
 *              properties:
 *                id:
 *                  description: ID of the updated item
 *                  type: string
 *                data:
 *                  description: Data of the updated item
 *                  type: object
 *                  $ref: "#/components/schemas/Pujari_applications"
 *              required:
 *                - id
 *      responses:
 *        200:
 *          description: The item data was successfully updated
 *          content:
 *            application/json:
 *              schema:
 *                $ref: "#/components/schemas/Pujari_applications"
 *        400:
 *          description: Invalid ID supplied
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        404:
 *          description: Item not found
 *        500:
 *          description: Some server error
 */

router.put(
  '/:id',
  wrapAsync(async (req, res) => {
    await Pujari_applicationsService.update(
      req.body.data,
      req.body.id,
      req.currentUser,
    );
    const payload = true;
    res.status(200).send(payload);
  }),
);

/**
 * @swagger
 *  /api/pujari_applications/{id}:
 *    delete:
 *      security:
 *        - bearerAuth: []
 *      tags: [Pujari_applications]
 *      summary: Delete the selected item
 *      description: Delete the selected item
 *      parameters:
 *        - in: path
 *          name: id
 *          description: Item ID to delete
 *          required: true
 *          schema:
 *            type: string
 *      responses:
 *        200:
 *          description: The item was successfully deleted
 *          content:
 *            application/json:
 *              schema:
 *                $ref: "#/components/schemas/Pujari_applications"
 *        400:
 *          description: Invalid ID supplied
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        404:
 *          description: Item not found
 *        500:
 *          description: Some server error
 */

router.delete(
  '/:id',
  wrapAsync(async (req, res) => {
    await Pujari_applicationsService.remove(req.params.id, req.currentUser);
    const payload = true;
    res.status(200).send(payload);
  }),
);

/**
 *  @swagger
 *  /api/pujari_applications:
 *    get:
 *      security:
 *        - bearerAuth: []
 *      tags: [Pujari_applications]
 *      summary: Get all pujari_applications
 *      description: Get all pujari_applications
 *      responses:
 *        200:
 *          description: Pujari_applications list successfully received
 *          content:
 *            application/json:
 *              schema:
 *                type: array
 *                items:
 *                  $ref: "#/components/schemas/Pujari_applications"
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        404:
 *          description: Data not found
 *        500:
 *          description: Some server error
 */

router.get(
  '/',
  wrapAsync(async (req, res) => {
    try{
      const payload = await Pujari_applicationsDBApi.findAll(req.query);
      res.status(200).send(payload);
    } catch (err) {
      console.log(err);
    }
  }),
);

router.get('/autocomplete', async (req, res) => {
  const payload = await Pujari_applicationsDBApi.findAllAutocomplete(
    req.query.query,
    req.query.limit,
  );
  res.status(200).send(payload);
});

/**
 * @swagger
 *  /api/pujari_applications/{id}:
 *    get:
 *      security:
 *        - bearerAuth: []
 *      tags: [Pujari_applications]
 *      summary: Get selected item
 *      description: Get selected item
 *      parameters:
 *        - in: path
 *          name: id
 *          description: ID of item to get
 *          required: true
 *          schema:
 *            type: string
 *      responses:
 *        200:
 *          description: Selected item successfully received
 *          content:
 *            application/json:
 *              schema:
 *                $ref: "#/components/schemas/Pujari_applications"
 *        400:
 *          description: Invalid ID supplied
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        404:
 *          description: Item not found
 *        500:
 *          description: Some server error
 */

router.get(
  '/:id',
  wrapAsync(async (req, res) => {
    const payload = await Pujari_applicationsDBApi.findBy({
      id: req.params.id,
    });
    res.status(200).send(payload);
  }),
);

router.use('/', require('../helpers').commonErrorHandler);

module.exports = router;
