const express = require('express');

const SamagriService = require('../db/api/samagri');
const SamagriDBApi = require('../db/api/samagri');
const wrapAsync = require('../helpers').wrapAsync;

const router = express.Router();

/**
 *  @swagger
 *  components:
 *    schemas:
 *      Samagri:
 *        type: object
 *        properties:

 *          name:
 *            type: string
 *            default: name
 *          description:
 *            type: string
 *            default: description

 *          standard_qty:
 *            type: integer
 *            format: int64
 *          price_standard_qty:
 *            type: integer
 *            format: int64
 *          units_in_stock:
 *            type: integer
 *            format: int64

 *          
 */

/**
 *  @swagger
 * tags:
 *   name: Samagri
 *   description: The Samagri managing API
 */

/**
 *  @swagger
 *  /api/samagri:
 *    post:
 *      security:
 *        - bearerAuth: []
 *      tags: [Samagri]
 *      summary: Add new item
 *      description: Add new item
 *      requestBody:
 *        required: true
 *        content:
 *          application/json:
 *            schema:
 *              properties:
 *                data:
 *                  description: Data of the updated item
 *                  type: object
 *                  $ref: "#/components/schemas/Samagri"
 *      responses:
 *        200:
 *          description: The item was successfully added
 *          content:
 *            application/json:
 *              schema:
 *                $ref: "#/components/schemas/Samagri"
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        405:
 *          description: Invalid input data
 *        500:
 *          description: Some server error
 */

router.post('/', async (req, res) => {
  console.log('req', req);
  await SamagriService.create(
    req.body.data,
    req.currentUser,
    true,
    req.headers.referer,
  );
  const payload = true;
  console.log('payload', payload);
  res.status(200).send(payload);
});

/**
 *  @swagger
 *  /api/samagri/{id}:
 *    put:
 *      security:
 *        - bearerAuth: []
 *      tags: [Samagri]
 *      summary: Update the data of the selected item
 *      description: Update the data of the selected item
 *      parameters:
 *        - in: path
 *          name: id
 *          description: Item ID to update
 *          required: true
 *          schema:
 *            type: string
 *      requestBody:
 *        description: Set new item data
 *        required: true
 *        content:
 *          application/json:
 *            schema:
 *              properties:
 *                id:
 *                  description: ID of the updated item
 *                  type: string
 *                data:
 *                  description: Data of the updated item
 *                  type: object
 *                  $ref: "#/components/schemas/Samagri"
 *              required:
 *                - id
 *      responses:
 *        200:
 *          description: The item data was successfully updated
 *          content:
 *            application/json:
 *              schema:
 *                $ref: "#/components/schemas/Samagri"
 *        400:
 *          description: Invalid ID supplied
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        404:
 *          description: Item not found
 *        500:
 *          description: Some server error
 */

router.put(
  '/:id',
  wrapAsync(async (req, res) => {
    console.log('req', req);
    await SamagriService.update(req.body.id, req.body.data, req.currentUser);
    const payload = true;
    console.log('payload', payload);
    res.status(200).send(payload);
  }),
);

/**
 * @swagger
 *  /api/samagri/{id}:
 *    delete:
 *      security:
 *        - bearerAuth: []
 *      tags: [Samagri]
 *      summary: Delete the selected item
 *      description: Delete the selected item
 *      parameters:
 *        - in: path
 *          name: id
 *          description: Item ID to delete
 *          required: true
 *          schema:
 *            type: string
 *      responses:
 *        200:
 *          description: The item was successfully deleted
 *          content:
 *            application/json:
 *              schema:
 *                $ref: "#/components/schemas/Samagri"
 *        400:
 *          description: Invalid ID supplied
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        404:
 *          description: Item not found
 *        500:
 *          description: Some server error
 */

router.delete(
  '/:id',
  wrapAsync(async (req, res) => {
    console.log('req.params.id', req.params.id);
    console.log('req.currentUser', req.currentUser);
    await SamagriService.remove(req.params.id, req.currentUser);
    const payload = true;
    console.log('payload', payload);
    res.status(200).send(payload);
  }),
);

/**
 *  @swagger
 *  /api/samagri:
 *    get:
 *      security:
 *        - bearerAuth: []
 *      tags: [Samagri]
 *      summary: Get all samagri
 *      description: Get all samagri
 *      responses:
 *        200:
 *          description: Samagri list successfully received
 *          content:
 *            application/json:
 *              schema:
 *                type: array
 *                items:
 *                  $ref: "#/components/schemas/Samagri"
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        404:
 *          description: Data not found
 *        500:
 *          description: Some server error
 */

router.get(
  '/',
  wrapAsync(async (req, res) => {
    console.log('req.query', req.query);
    const payload = await SamagriDBApi.findAll(req.query);
    console.log('payload', payload);
    res.status(200).send(payload);
  }),
);

router.get('/autocomplete', async (req, res) => {
  console.log('req.query', req.query);
  const payload = await SamagriDBApi.findAllAutocomplete(
    req.query.query,
    req.query.limit,
  );
  console.log('payload', payload);
  res.status(200).send(payload);
});

/**
 * @swagger
 *  /api/samagri/{id}:
 *    get:
 *      security:
 *        - bearerAuth: []
 *      tags: [Samagri]
 *      summary: Get selected item
 *      description: Get selected item
 *      parameters:
 *        - in: path
 *          name: id
 *          description: ID of item to get
 *          required: true
 *          schema:
 *            type: string
 *      responses:
 *        200:
 *          description: Selected item successfully received
 *          content:
 *            application/json:
 *              schema:
 *                $ref: "#/components/schemas/Samagri"
 *        400:
 *          description: Invalid ID supplied
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        404:
 *          description: Item not found
 *        500:
 *          description: Some server error
 */

router.get(
  '/:id',
  wrapAsync(async (req, res) => {
    console.log('req.params.id', req.params.id);
    const payload = await SamagriDBApi.findBy({ id: req.params.id });
    console.log('payload', payload);
    res.status(200).send(payload);
  }),
);

router.use('/', require('../helpers').commonErrorHandler);

module.exports = router;
